﻿using Lab3.Database.Context;
using Lab3.Database.MappingProfiles;
using Lab3.Database.Repository.Implementations;
using Lab3.Database.Repository.Interfaces;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;

namespace Lab3.Database.Extensions
{
    public static class DiExtension
    {
        public static IServiceCollection AddDatabase(
            this IServiceCollection services,
            IConfiguration configuration)
        {
            _ = services.AddDbContextPool<COPContext>(
                dbContextOptions =>
                {
                    AppContext.SetSwitch("Npgsql.EnableLegacyTimestampBehavior", true);
                    AppContext.SetSwitch("Npgsql.DisableDateTimeInfinityConversions", true);

                    _ = dbContextOptions.UseNpgsql(configuration.GetConnectionString("COPDataBase"));
                    _ = dbContextOptions.ConfigureWarnings(warnings => { });
                }
            );

            return services;
        }

        public static IServiceCollection AddDbMapping(
            this IServiceCollection services)
        {
            services.AddAutoMapper(typeof(StudentMappingProfile));
            services.AddAutoMapper(typeof(SessionMappingProfile));

            return services;
        }

        public static IServiceCollection AddRepositories(
            this IServiceCollection services)
        {
            services.AddScoped<IStudentRepository, StudentRepository>();
            services.AddScoped<IEducationFormRepository, EducationFormRepository>();

            return services;
        }
    }
}
