﻿using Lab3.Database.DTO;
using Lab3.Database.Models;
using Lab3.Database.Repository.Interfaces;

namespace Lab3.Forms
{
    public partial class CreateForm : Form
    {
        private readonly IStudentRepository _studentRepository;

        private readonly IEducationFormRepository _educationFormRepository;

        private readonly Guid? _updatedStudentGuid = null;

        public CreateForm(
            IStudentRepository studentRepository,
            IEducationFormRepository educationFormRepository,
            Guid? updatedStudentGuid)
        {
            _studentRepository = studentRepository;
            _educationFormRepository = educationFormRepository;
            
            _updatedStudentGuid = updatedStudentGuid;

            InitializeComponent();
        }

        private async void CreateForm_LoadAsync(object sender, EventArgs e)
        {
            FormSelector.Fill(await _educationFormRepository.Get());
            StartEducationDataPicker.DateStart = DateTime.Now.AddYears(-6);
            StartEducationDataPicker.DateEnd = DateTime.Now;

            if (_updatedStudentGuid == null)
            {
                return;
            }
            var student = await _studentRepository.GetAsync(_updatedStudentGuid.Value);

            NameTextBox.Text = student!.Name;
            StartEducationDataPicker.Value = student.StartEducation;
            FormSelector.ComboBoxSelectedValue = student.EducationForm;

            session1Score.Value = student.StudentSessions
                .FirstOrDefault(s => s.Number == 1)?.Score ?? decimal.Zero;
            session2Score.Value = student.StudentSessions
                .FirstOrDefault(s => s.Number == 2)?.Score ?? decimal.Zero;
            session3Score.Value = student.StudentSessions
                .FirstOrDefault(s => s.Number == 3)?.Score ?? decimal.Zero;
            session4Score.Value = student.StudentSessions
                .FirstOrDefault(s => s.Number == 4)?.Score ?? decimal.Zero;
            session5Score.Value = student.StudentSessions
                .FirstOrDefault(s => s.Number == 5)?.Score ?? decimal.Zero;
            session6Score.Value = student.StudentSessions
                .FirstOrDefault(s => s.Number == 6)?.Score ?? decimal.Zero;
        }

        private async void ButtonSave_ClickAsync(object sender, EventArgs e)
        {
            try
            {
                if (string.IsNullOrEmpty(NameTextBox.Text)
                || string.IsNullOrEmpty(FormSelector.ComboBoxSelectedValue))
                {
                    throw new Exception();
                }

                StudentDTO student = new()
                {
                    Id = _updatedStudentGuid ?? Guid.NewGuid(),
                    Name = NameTextBox.Text,
                    StartEducation = StartEducationDataPicker.Value,
                    EducationForm = FormSelector.ComboBoxSelectedValue,
                    StudentSessions = [
                        new(){
                        Score = session1Score.Value,
                        Number = 1,
                    },
                    new(){
                        Score = session2Score.Value,
                        Number = 2,
                    },
                    new(){
                        Score = session3Score.Value,
                        Number = 3,
                    },
                    new(){
                        Score = session4Score.Value,
                        Number = 4,
                    },
                    new(){
                        Score = session5Score.Value,
                        Number = 5,
                    },
                    new(){
                        Score = session6Score.Value,
                        Number = 6,
                    },
                ],
                };

                if (_updatedStudentGuid != null)
                {
                    _ = await _studentRepository.UpdateAsync(student);
                }
                else
                {
                    _ = await _studentRepository.CreateAsync(student);
                }
                Close();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
    }
}
